@@ -7,7 +7,7 @@
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
 #
-# Copyright 2004-2023 Univention GmbH
+# Copyright 2004-2024 Univention GmbH
 #
 # https://www.univention.de/
 #
@@ -92,6 +92,17 @@ def get_mapping(configbasename='connector'):
 def create_mapping(configbasename='connector'):
     def connector(string):
         return string % (configbasename,)
+
+    global_allow_subtree_ad = []
+    global_allow_subtree_ucs = []
+
+    for key in configRegistry:
+        if key.startswith(connector('%s/ad/mapping/allowsubtree')):
+            if key.endswith('/ad'):
+                global_allow_subtree_ad.append(configRegistry[key])
+            elif key.endswith('/ucs'):
+                global_allow_subtree_ucs.append(configRegistry[key])
+
     global_ignore_subtree = [
         'cn=univention,%(ldap/base)s' % configRegistry,
         'cn=policies,%(ldap/base)s' % configRegistry,
@@ -126,10 +137,31 @@ def create_mapping(configbasename='connector'):
     user_ignore_filter = '(|{}{}{})'.format('(userAccountControl=2080)', user_ignore_filter, user_ignore_list)
 
     ignore_filter_parts = '(groupType=-2147483643)(groupType=4)(univentionGroupType=-2147483643)(univentionGroupType=4)'
+    group_ignore_filter = configRegistry.get(connector('%s/ad/mapping/group/ignorefilter'), '')
+    if group_ignore_filter and not group_ignore_filter.startswith('('):
+        group_ignore_filter = f'({group_ignore_filter})'
     if configRegistry.is_false(connector('%s/ad/mapping/group/grouptype'), False):
         ignore_filter_parts += '(sambaGroupType=5)(groupType=5)'
     ignore_filter_parts += ignore_filter_from_attr('cn', connector('%s/ad/mapping/group/ignorelist'))
-    group_ignore_filter = f'(|{ignore_filter_parts})'
+    group_ignore_filter = f'(|{group_ignore_filter}{ignore_filter_parts})'
+
+    computer_ignore_filter = configRegistry.get(connector('%s/ad/mapping/windowscomputer/ignorefilter'), '')
+    if computer_ignore_filter and not computer_ignore_filter.startswith('('):
+        computer_ignore_filter = f'({computer_ignore_filter})'
+    computer_ignore_parts = ignore_filter_from_attr('cn', connector('%s/ad/mapping/windowscomputer/ignorelist'))
+    computer_ignore_filter = f'(|{computer_ignore_filter}{computer_ignore_parts})' if computer_ignore_filter or computer_ignore_parts else ''
+
+    container_ignore_attrs = ignore_filter_from_attr('cn', connector('%s/ad/mapping/container/ignorelist'), 'mail,kerberos')
+    container_ignore_filter = configRegistry.get(connector('%s/ad/mapping/container/ignorefilter'), '')
+    if container_ignore_filter and not container_ignore_filter.startswith('('):
+        container_ignore_filter = f'({container_ignore_filter})'
+    container_ignore_filter = f'(|{container_ignore_filter}{container_ignore_attrs})' if container_ignore_filter or container_ignore_attrs else ''
+
+    ou_ignore_attrs = ignore_filter_from_attr('ou', connector('%s/ad/mapping/ou/ignorelist'))
+    ou_ignore_filter = configRegistry.get(connector('%s/ad/mapping/ou/ignorefilter'), '')
+    if ou_ignore_filter and not ou_ignore_filter.startswith('('):
+        ou_ignore_filter = f'({ou_ignore_filter})'
+    ou_ignore_filter = f'(|{ou_ignore_filter}{ou_ignore_attrs})' if ou_ignore_filter or ou_ignore_attrs else ''
 
     ad_mapping = {
         'user': univention.connector.property(
@@ -141,6 +173,7 @@ def create_mapping(configbasename='connector'):
             scope='sub',
             con_search_filter='(&(objectClass=user)(!objectClass=computer))',
             match_filter='(|(&(objectClass=posixAccount)(objectClass=sambaSamAccount))(objectClass=user))',
+            allow_subtree=global_allow_subtree_ucs + global_allow_subtree_ad,
             ignore_filter=user_ignore_filter or None,
             ignore_subtree=global_ignore_subtree,
             con_create_objectclass=['top', 'user', 'person', 'organizationalPerson'],
@@ -309,6 +342,7 @@ def create_mapping(configbasename='connector'):
             ucs_module='groups/group',
             sync_mode=configRegistry.get(connector('%s/ad/mapping/group/syncmode'), configRegistry.get(connector('%s/ad/mapping/syncmode'))),
             scope='sub',
+            allow_subtree=global_allow_subtree_ucs + global_allow_subtree_ad,
             ignore_filter=group_ignore_filter or None,
             ignore_subtree=global_ignore_subtree,
             con_search_filter='objectClass=group',
@@ -398,8 +432,9 @@ def create_mapping(configbasename='connector'):
             con_search_filter='(&(objectClass=computer)(userAccountControl:1.2.840.113556.1.4.803:=4096))',
             # ignore_filter='userAccountControl=4096',
             match_filter='(|(&(objectClass=univentionWindows)(!(univentionServerRole=windows_domaincontroller)))(objectClass=computer)(objectClass=univentionMemberServer)(objectClass=univentionUbuntuClient)(objectClass=univentionLinuxClient)(objectClass=univentionMacOSClient))',
+            allow_subtree=global_allow_subtree_ucs + global_allow_subtree_ad,
             ignore_subtree=global_ignore_subtree,
-            ignore_filter=ignore_filter_from_attr('cn', connector('%s/ad/mapping/windowscomputer/ignorelist')) or None,
+            ignore_filter=computer_ignore_filter or None,
             con_create_objectclass=['top', 'computer'],
             con_create_attributes=[('userAccountControl', [b'4096'])],
             attributes={
@@ -438,7 +473,8 @@ def create_mapping(configbasename='connector'):
             sync_mode=configRegistry.get(connector('%s/ad/mapping/container/syncmode'), configRegistry.get(connector('%s/ad/mapping/syncmode'))),
             scope='sub',
             con_search_filter='(|(objectClass=container)(objectClass=builtinDomain))',  # builtinDomain is cn=builtin (with group cn=Administrators)
-            ignore_filter=ignore_filter_from_attr('cn', connector('%s/ad/mapping/container/ignorelist'), 'mail,kerberos') or None,
+            allow_subtree=global_allow_subtree_ucs + global_allow_subtree_ad,
+            ignore_filter=container_ignore_filter or None,
             ignore_subtree=global_ignore_subtree,
             post_ucs_modify_functions=[
                 univention.connector.ad.set_univentionObjectFlag_to_synced,
@@ -464,7 +500,8 @@ def create_mapping(configbasename='connector'):
             sync_mode=configRegistry.get(connector('%s/ad/mapping/ou/syncmode'), configRegistry.get(connector('%s/ad/mapping/syncmode'))),
             scope='sub',
             con_search_filter='objectClass=organizationalUnit',
-            ignore_filter=ignore_filter_from_attr('ou', connector('%s/ad/mapping/ou/ignorelist')) or None,
+            allow_subtree=global_allow_subtree_ucs + global_allow_subtree_ad,
+            ignore_filter=ou_ignore_filter or None,
             ignore_subtree=global_ignore_subtree,
             post_ucs_modify_functions=[
                 univention.connector.ad.set_univentionObjectFlag_to_synced,
@@ -487,6 +524,14 @@ def create_mapping(configbasename='connector'):
         ),
     }
 
+    # allow filter
+    for obj_type in ad_mapping.keys():
+        allow_filter = configRegistry.get(connector(f'%s/ad/mapping/{obj_type}/allowfilter'), '')
+        if allow_filter:
+            if not allow_filter.startswith('('):
+                allow_filter = f'({allow_filter})'
+            ad_mapping[obj_type].allow_filter = allow_filter
+
     # users
     if configRegistry.is_false(connector('%s/ad/mapping/user/exchange'), True):
         ad_mapping['user'].post_attributes.pop('Exchange-Homeserver')
