@@ -7,7 +7,7 @@
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
 #
-# Copyright 2018-2023 Univention GmbH
+# Copyright 2018-2024 Univention GmbH
 #
 # https://www.univention.de/
 #
@@ -74,6 +74,7 @@ class ad(univention.connector.ad.ad):
             c.execute("DELETE from GUIDS where id = ?", (guid_id,))
             cache_db.commit()
         cache_db.close()
+        os.chmod(cache_filename, 640)
 
     def _add_object_to_rejected(self, ad_dn, usn):
         state_filename = '/etc/univention/%s/internal.sqlite' % CONFIGBASENAME
@@ -82,6 +83,7 @@ class ad(univention.connector.ad.ad):
         c.execute("INSERT OR REPLACE INTO 'AD rejected' (key, value) VALUES (?, ?);", (usn, ad_dn))
         db.commit()
         db.close()
+        os.chmod(state_filename, 640)
 
     def resync(self, ad_dns=None, ldapfilter=None, ldapbase=None):
         treated_dns = []
@@ -150,11 +152,11 @@ class ad(univention.connector.ad.ad):
 
 
 if __name__ == '__main__':
-    parser = ArgumentParser()
+    parser = ArgumentParser(description="Resync object from AD to UCS")
     parser.add_argument("-f", "--filter", dest="ldapfilter", help="LDAP search filter")
     parser.add_argument("-b", "--base", dest="ldapbase", help="LDAP search base")
-    parser.add_argument("-c", "--configbasename", help="", metavar="CONFIGBASENAME", default="connector")
-    parser.add_argument('dn')
+    parser.add_argument("-c", "--configbasename", help="Config basename", metavar="CONFIGBASENAME", default="connector")
+    parser.add_argument("dn", nargs='?', default=None, help="Active Directory DN to resync")
     options = parser.parse_args()
 
     CONFIGBASENAME = options.configbasename
